@@ -20,10 +20,12 @@
 package org.apache.royale.compiler.internal.scopes;
 
 import org.apache.royale.compiler.common.DependencyType;
+import org.apache.royale.compiler.config.CompilerDiagnosticsConstants;
 import org.apache.royale.compiler.constants.IASLanguageConstants;
 import org.apache.royale.compiler.definitions.IDefinition;
 import org.apache.royale.compiler.definitions.IInterfaceDefinition;
 import org.apache.royale.compiler.definitions.INamespaceDefinition;
+import org.apache.royale.compiler.definitions.ITypeDefinition;
 import org.apache.royale.compiler.definitions.references.IResolvedQualifiersReference;
 import org.apache.royale.compiler.internal.definitions.AmbiguousDefinition;
 import org.apache.royale.compiler.internal.definitions.ClassDefinitionBase;
@@ -31,6 +33,7 @@ import org.apache.royale.compiler.internal.definitions.ConstantDefinition;
 import org.apache.royale.compiler.internal.definitions.TypeDefinitionBase;
 import org.apache.royale.compiler.internal.projects.CompilerProject;
 import org.apache.royale.compiler.projects.ICompilerProject;
+import org.apache.royale.compiler.units.ICompilationUnit;
 
 import com.google.common.collect.MapMaker;
 
@@ -130,15 +133,27 @@ public class ASScopeCache
      * @param dt Which type of dependency to introduce when we do the lookup
      * @return The IDefinition for the property, or null if it wasn't found
      */
-    IDefinition findProperty(String name, DependencyType dt)
+    IDefinition findProperty(String name, DependencyType dt, boolean favorTypes)
     {
         ConcurrentMap<String, IDefinition> map = getScopeChainMap();
 
         IDefinition result = map.get(name);
         if (result != null)
         {
-            assert result.isInProject(project);
             // We found a cached result - we're done
+        	// after making sure it has a dependency
+        	if (result instanceof ITypeDefinition)
+        	{
+	        	ICompilationUnit from = scope.getFileScope().getCompilationUnit();
+	            assert result.isInProject(project);
+	            
+	            String qname = result.getQualifiedName();
+	            ICompilationUnit to = ((ASProjectScope)project.getScope()).getCompilationUnitForDefinition(result);
+	            if (to == null && !(qname.contentEquals("void") || qname.contentEquals("*")))
+	            	System.out.println("No compilation unit for " + qname);	
+	            if (to != null)
+	            	project.addDependency(from, to, dt, qname);
+        	}
             return result;
         }
 
@@ -147,6 +162,7 @@ public class ASScopeCache
         // the benefit is that we avoid any sort of locking, which was proving expensive (time wise,
         // and memory wise).
 
+        boolean wasAmbiguous = false;
         IDefinition def = null;
         Set<INamespaceDefinition> namespaceSet = scope.getNamespaceSetForName(project, name);
         // Look for the definition in the scope
@@ -163,7 +179,8 @@ public class ASScopeCache
                 assert def.isInProject(project);
                 break;
             default:
-                IDefinition d = AmbiguousDefinition.resolveAmbiguities(project, defs);
+            	wasAmbiguous = true;
+                IDefinition d = AmbiguousDefinition.resolveAmbiguities(project, defs, favorTypes);
                 if (d != null)
                     def = d;
                 else {
@@ -185,7 +202,7 @@ public class ASScopeCache
             // If the dependency type is null we can't cache the name
             // resolution result, because the name resolution cache will not
             // be properly invalidated when the file containing the definition changes.
-            if (dt != null)
+            if (dt != null && !wasAmbiguous)
             {
                 result = map.putIfAbsent(name, def);
                 if (result == null)
@@ -205,6 +222,8 @@ public class ASScopeCache
         ConcurrentMap<String, IDefinition> map = findPropCache != null ? findPropCache.get() : null;
         if (map == null)
         {
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache waiting for lock in getScopeChainMap");
             synchronized (this)
             {
                 // Check again, in case another thread updated the map first
@@ -215,6 +234,8 @@ public class ASScopeCache
                     findPropCache = new SoftReference<ConcurrentMap<String, IDefinition>>(map);
                 }
             }
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in getScopeChainMap");
         }
         return map;
     }
@@ -224,6 +245,8 @@ public class ASScopeCache
         ConcurrentMap<QName, IDefinition> map = findPropQualifiedCache != null ? findPropQualifiedCache.get() : null;
         if (map == null)
         {
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache waiting for lock in getQualifiedScopeChainMap");
             synchronized (this)
             {
                 // Check again, in case another thread updated the map first
@@ -234,6 +257,8 @@ public class ASScopeCache
                     findPropQualifiedCache = new SoftReference<ConcurrentMap<QName, IDefinition>>(map);
                 }
             }
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in getQualifiedScopeChainMap");
         }
         return map;
     }
@@ -285,7 +310,7 @@ public class ASScopeCache
                 break;
 
             default:
-                IDefinition d = AmbiguousDefinition.resolveAmbiguities(project, defs);
+                IDefinition d = AmbiguousDefinition.resolveAmbiguities(project, defs, false);
                 if (d != null)
                     def = d;
                 else
@@ -352,7 +377,7 @@ public class ASScopeCache
                 assert def.isInProject(project);
                 break;
             default:
-                IDefinition d = AmbiguousDefinition.resolveAmbiguities(project, defs);
+                IDefinition d = AmbiguousDefinition.resolveAmbiguities(project, defs, false);
                 if (d != null)
                     def = d;
                 else
@@ -402,6 +427,8 @@ public class ASScopeCache
         ConcurrentMap<String, Set<INamespaceDefinition>> map = namespacesForNameCache != null ? namespacesForNameCache.get() : null;
         if (map == null)
         {
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache waiting for lock in getNamespacesForNameMap");
             synchronized (this)
             {
                 // Check again, in case another thread updated the map first
@@ -412,6 +439,8 @@ public class ASScopeCache
                     namespacesForNameCache = new SoftReference<ConcurrentMap<String, Set<INamespaceDefinition>>>(map);
                 }
             }
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in getNamespacesForNameMap");
         }
         return map;
     }
@@ -421,6 +450,8 @@ public class ASScopeCache
         ConcurrentMap<IResolvedQualifiersReference, IDefinition> map = multinameLookupCache != null ? multinameLookupCache.get() : null;
         if (map == null)
         {
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache waiting for lock in getMultinameLookupMap");
             synchronized (this)
             {
                 // Check again, in case another thread updated the map first
@@ -431,6 +462,8 @@ public class ASScopeCache
                     multinameLookupCache = new SoftReference<ConcurrentMap<IResolvedQualifiersReference, IDefinition>>(map);
                 }
             }
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in getMultinameLookupMap");
         }
         return map;
     }
@@ -440,6 +473,8 @@ public class ASScopeCache
         ConcurrentMap<IDefinition, Object> map = constValueLookupCache != null ? constValueLookupCache.get() : null;
         if (map == null)
         {
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache waiting for lock in getConstantValueLookupMap");
             synchronized (this)
             {
                 // Check again, in case another thread updated the map first
@@ -450,6 +485,8 @@ public class ASScopeCache
                     constValueLookupCache = new SoftReference<ConcurrentMap<IDefinition, Object>>(map);
                 }
             }
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in getConstantValueLookupMap");
         }
         return map;
     }
@@ -459,6 +496,8 @@ public class ASScopeCache
         Set<IASLanguageConstants.BuiltinType> set = builtinTypeDependencyCache != null ? builtinTypeDependencyCache.get() : null;
         if (set == null)
         {
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache waiting for lock in getBuiltinTypeMap");
             synchronized (this)
             {
                 // Check again, in case another thread updated the set first
@@ -469,6 +508,8 @@ public class ASScopeCache
                     builtinTypeDependencyCache = new SoftReference<Set<IASLanguageConstants.BuiltinType>>(set);
                 }
             }
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in getBuiltinTypeMap");
         }
         return set;
     }
@@ -494,14 +535,22 @@ public class ASScopeCache
         Boolean valueObject = needsEventDispatcherCache;
         if (valueObject != null)
             return valueObject.booleanValue();
+    	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+    		System.out.println("ASScopeCache waiting for lock in needsEventDispatcher");
         synchronized (this)
         {
             // Check again, in case another thread updated the value first
             valueObject = needsEventDispatcherCache;
             if (valueObject != null)
+            {
+            	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+            		System.out.println("ASScopeCache done with lock in needsEventDispatcher");
                 return valueObject.booleanValue();
+            }
             boolean computedValue = ((ClassDefinitionBase)scope.getDefinition()).computeNeedsEventDispatcher(project);
             needsEventDispatcherCache = computedValue;
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in needsEventDispatcher");
             return computedValue;
         }
     }
@@ -516,15 +565,23 @@ public class ASScopeCache
         if( interfs != null )
             return interfs;
 
+    	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+    		System.out.println("ASScopeCache waiting for lock in resolveInterfaces");
         synchronized (this)
         {
             // check again in case another thread updated the value first
             interfs = interfacesCache != null ? interfacesCache.get() : null;
             if( interfs != null )
+            {
+            	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+            		System.out.println("ASScopeCache done with lock in resolveInterfaces");
                 return interfs;
+            }
 
             interfs = ((TypeDefinitionBase)scope.getDefinition()).resolveInterfacesImpl(project);
             interfacesCache = new SoftReference<IInterfaceDefinition[]>(interfs);
+        	if ((CompilerDiagnosticsConstants.diagnostics & CompilerDiagnosticsConstants.ASSCOPECACHE) == CompilerDiagnosticsConstants.ASSCOPECACHE)
+        		System.out.println("ASScopeCache done with lock in resolveInterfaces");
             return interfs;
         }
     }
